(ns com.aurellem.title
  (:use (com.aurellem gb-driver vbm)))

(defn delayed-key
  ([key delay total]
     (concat (repeat delay []) [key] (repeat (- total delay 1) [])))
  ([key total]
     (delayed-key key (dec total) total)))

(defn no-action [length]
  (repeat length []))

(defn start-summary []
  (nth (registers) 2))

(defn common-initial-elements [baseline moves]
  (loop [common 0 b baseline m moves]
    (if (empty? m) common
        (if (= (first b) (first m))
          (recur (inc common) (rest b) (rest m))
          common))))
    
(defn earliest-press
  [start-frame
   end-frame
   key
   summary-fn]
  (let [action-length (- end-frame start-frame)
        baseline (no-action action-length)]
    (print "establishing baseline...")
    (play-moves start-frame baseline)
    (let [bad-value (summary-fn)]
      (println bad-value)
      (loop [n 0]
        (let [moves (delayed-key key n action-length)
              header-length
              (common-initial-elements moves baseline)]
          (print "length" (inc n) "...")
          (without-saves
           (play-moves
            (+ start-frame header-length)
            (drop header-length moves)))
          (let [result (summary-fn)]
            (println result)
            (if (not= result bad-value)
              (let [keys (delayed-key key (inc n))]
                  (play-moves start-frame keys)
                  keys)
              (recur (inc n)))))))))


(defn search-first
  [start-frame
   baseline
   gen-move-fn
   summary-fn]
  (print "establishing baseline...")
  (play-moves start-frame baseline)
  (let [bad-value (summary-fn)]
    (println bad-value)
    (loop [n 0]
      (let [trial-moves (gen-move-fn n)
            header-length
            (common-initial-elements trial-moves baseline)]
        (print "length" (inc n) "...")
        (without-saves
         (play-moves
          (+ start-frame header-length)
          (drop header-length trial-moves)))
        (let [result (summary-fn)]
          (println result)
          (if (not= result bad-value)
            (let [keys (take (inc n) trial-moves)]
              (play-moves start-frame keys)
              keys)
            (recur (inc n))))))))

(defn title-search
  [start-frame
   end-frame
   key
   summary-fn]
  (let [action-length (- end-frame start-frame)]
    (search-first
     start-frame
     (no-action action-length)
     (fn [n] (delayed-key key n action-length))
     summary-fn)))

(defn gen-title []
  (let [start0 (no-action 300)]
    (play-moves 0 start0)
    (let [start->first-press
          (title-search (frame) (+ 50 (frame)) [:a] start-summary)
          first-press->second-press
          (title-search (frame) (+ 100 (frame)) [:start] start-summary)
          second-press->third-press
          (title-search (frame) (+ 151 (frame)) [:a] start-summary)
          new-game
          (title-search (frame) (+ 151 (frame)) [:a] start-summary)]
      (concat
       start0
       start->first-press
       first-press->second-press
       second-press->third-press
       new-game))))
  
(def title
  [[] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [ :a] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [:start] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [ :a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
   [] [] [] [] [] [ :a]])


(require '(clojure [zip :as zip]))